-- This file contains mocks/stubs, general test utilities, and a test runner.
-- Test suites are in separate files and included (via `require()`) below.

--- Mocks/stubs

require('test.mocks')

--- Require addon

require('../flamingloot')

--- Test utils

local indent = 4 -- default for when called inside tests

function tprint(msg, ind)
  -- print with indentation level
  ind = ind or indent
  msg = string.gsub(msg, '\n', '\n'..string.rep(' ', ind))
  print(string.rep(' ', ind)..msg)
end

function printTable(t)
  for k, v in pairs(t) do
    if type(v) == "table" then
      tprint(k..': ')
      indent = indent + 2
      printTable(v)
      indent = indent - 2
    else
      tprint(k..': '..tostring(v))
    end
  end
end

--- Test suites

suites = {} -- set up global test table that suites refer to

require('test.util')

--- Test runner

local function before()
  -- initialize mocks
  FlamingLoot.Print = mock.Print
  mock:_LogToDB()
end

local function beforeEach()
  FlamingLoot:OnInitialize()
  FlamingLoot:OnEnable()
end

local EXCLUDE = {
  before = true,
  beforeEach = true,
  after = true,
  afterEach = true
}

RED = "\027[00;31m"
GREEN = "\027[00;32m"
BLUE = "\027[00;34m"
RESET = "\027[00m"

local FAIL = RED..'X fail'..RESET
local PASS = GREEN..'✓ pass'..RESET

local passed = 0
local failed = 0

function runSuite(name, suite)
  tprint(BLUE..name..RESET, 0)

  -- before all hook
  if suite.before then
    tprint('before', 2)
    suite.before()
  end

  -- run tests
  for test_name, test in pairs(suite) do
    if not EXCLUDE[test_name] then
      -- beforeEach
      if suite.beforeEach then
        suite.beforeEach()
      end
      tprint(BLUE..test_name..RESET, 2)
      -- test
      local pass, err = pcall(test)
      if not pass then
        failed = failed + 1
        tprint(FAIL, 2)
        print(err)
        print(debug.traceback())
      else
        passed = passed + 1
        tprint(PASS, 2)
      end
      -- afterEach
      if suite.afterEach then
        suite.afterEach()
      end
      print(' ')
    end
  end

  -- after all hook
  if suite.after then
    tprint('after', 2)
    suite.after()
  end

end

--- Execute tests!

local start = os.time()
before()
for name, suite in pairs(suites) do
  beforeEach()
  runSuite(name, suite)
end
local testColor
if failed > 0 then
  testColor = RED
else
  testColor = GREEN
end
print(testColor..'Testing complete ('..(os.time() - start)..' s)'..
  ' (PASSED: '..passed..', FAILED: '..failed..')'..RESET)


